/** @file win_gui.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2014.03.24 -

	@brief [ gui for windows ]
*/

#include "win_gui.h"
#include "../../config.h"
#include "../../winfont.h"
#include "../../emu.h"
#include "../../emumsg.h"
#include "win_filebox.h"
#include "win_seldrvbox.h"
#include "../../utility.h"
#include "../../winmain.h"

namespace GUI_WIN
{
#ifdef GUI_TYPE_WINDOWS

#define TRIM_STRING_SIZE	72

#define UPDATE_RECENT_MENU(ID_RECENT, recent_path) { \
	bool flag = false; \
	HMENU hSubMenu = get_sub_menu(hMenu, ID_RECENT); \
	_TCHAR str[_MAX_PATH]; \
	if (hSubMenu) { \
		while(DeleteMenu(hSubMenu, 0, MF_BYPOSITION) != 0) {} \
		for(int i = 0; i < MAX_HISTORY; i++) { \
			if (!GetRecentFileStr(recent_path[i].path, recent_path[i].num, str, TRIM_STRING_SIZE)) break; \
			AppendMenu(hSubMenu, MF_STRING, ID_RECENT + i, str); \
			flag = true; \
		} \
		if(!flag) { \
			AppendMenu(hSubMenu, MF_GRAYED | MF_STRING, ID_RECENT, _("None")); \
		} \
	} \
}

#define UPDATE_MULTIVOLUME_MENU(drv, ID_D88_FILE_PATH, ID_SELECT_BANK) { \
	bool flag = false; \
	HMENU hSubMenu = get_sub_menu(hMenu, ID_SELECT_BANK); \
	_TCHAR str[32]; \
	if (hSubMenu) { \
		while(DeleteMenu(hSubMenu, 0, MF_BYPOSITION) != 0) {} \
		d88_file_t *d88_file = GetD88File(drv); \
		if(d88_file->bank_nums >= 1) { \
			for(int i = 0; i < d88_file->bank_nums; i++) { \
				GetMultiVolumeStr(i, d88_file->bank[i].name, str); \
				AppendMenu(hSubMenu \
					, MF_STRING | (d88_file->bank_nums <= 1 ? MF_GRAYED : 0) | (i == d88_file->cur_bank ? MF_CHECKED : 0) \
					, ID_SELECT_BANK + i, str); \
			} \
			flag = true; \
		} \
		if(!flag) { \
			AppendMenu(hSubMenu, MF_GRAYED | MF_STRING, ID_SELECT_BANK, _("None")); \
		} \
	} \
}

static HMENU get_sub_menu(HMENU hMenu, int id) {
	HMENU hSubMenu = NULL;
	int count = GetMenuItemCount(hMenu);
	for(int n=count-1; n>=0; n--) {
		hSubMenu = GetSubMenu(hMenu,n);
		if (!hSubMenu) continue;

		MENUITEMINFO mii;
		memset(&mii, 0, sizeof(mii));
		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_ID;
		if (!GetMenuItemInfo(hSubMenu, id, FALSE, &mii)) {
			hSubMenu = NULL;
			continue;
		}
		break;
	}
	return hSubMenu;
}

/****************************************

 GUI for windows class

 ****************************************/

GUI::GUI(int argc, char **argv, EMU *new_emu) : GUI_BASE(argc, argv, new_emu)
{
	hInstance = ::GetModuleHandle(NULL);

	font = new CFont();
	font->SetDefaultFont();

	configbox = new ConfigBox(hInstance, font, emu, &config);
	keybindbox = new KeybindBox(hInstance, font, emu);
	volumebox = new VolumeBox(hInstance, font, emu, &config);
	recvidbox = new RecVideoBox(hInstance, font, emu);
	aboutbox = new AboutBox(hInstance, font, emu);
#ifdef USE_DEBUG_SOUND_FILTER
	sndfilterbox = new SndFilterBox(hInstance, font, emu);
#endif

	KeybindControl::RegisterClass(hInstance);

	now_menu = false;
	now_menuloop = false;

}

GUI::~GUI()
{
#ifdef USE_DEBUG_SOUND_FILTER
	delete sndfilterbox;
#endif
	delete aboutbox;
	delete recvidbox;
	delete volumebox;
	delete keybindbox;
	delete configbox;
	delete font;

	KeybindControl::UnregisterClass(hInstance);
}

int GUI::CreateWidget(HWND new_hwnd, int width, int height)
{
	hWindow = new_hwnd;
	return 0;
}

int GUI::CreateMenu()
{
	ShowMenu();
	return 0;
}

void GUI::ShowMenu()
{
	if(!now_menu) {
		HMENU hMenu = LoadMenu((HINSTANCE)GetModuleHandle(0), MAKEINTRESOURCE(IDR_MENU1));
		translate_menu(hMenu, 0);
		SetMenu(hWindow, hMenu);
		now_menu = true;
	}
}

void GUI::HideMenu()
{
	if(now_menu) {
		HMENU hMenu = GetMenu(hWindow);
		SetMenu(hWindow, NULL);
		DestroyMenu(hMenu);
		now_menu = false;
	}
}

LRESULT GUI::ProcessEvent(UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	LRESULT result = GUI_BASE::ProcessEvent(iMsg, wParam, lParam);
	if (result != 1) return result;

	switch(iMsg) {
	case WM_INITMENUPOPUP:
//		if(emu) {
//			emu->mute_sound(true);
//			SystemPause(true);
//		}
		update_menu((HMENU)wParam, LOWORD(lParam));
		return 0;

	case WM_ENTERMENULOOP:
//		if(emu) {
//			SystemPause(true);
//		}
		now_menuloop = true;
		return 0;

	case WM_EXITMENULOOP:
		if(emu->is_fullscreen() && now_menuloop) {
			HideMenu();
		}
		if (emu) {
			emu->key_lost_focus();
//			emu->mute_sound(false);
			SystemPause(false);
		}
		now_menuloop = false;
		return 0;

	case WM_MOVE:
//		if(emu) {
//			emu->move_led();
//		}
		MoveLedBox();
		return 0;

	case WM_MOUSEMOVE:
		if(emu->is_fullscreen() && !now_menuloop) {
			POINTS p = MAKEPOINTS(lParam);
			if(p.y == 0) {
				ShowMenu();
			}
			else if(p.y > 32) {
				HideMenu();
			}
		}
		return 0;

	}
	return 1;
}

void GUI::ScreenModeChanged(bool fullscreen)
{
	if (fullscreen) {
		HideMenu();
	} else {
		ShowMenu();
	}
}

void GUI::SetFocusToMainWindow()
{
	::SetFocus(hWindow);
}

bool GUI::ShowRecordVideoDialog(int num)
{
	bool rc = false;
	PostEtSystemPause(true);
	if (recvidbox->Show(hWindow) == IDOK) {
		PostEtStartRecordVideo(num);
		rc = true;
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return rc;
}

#ifdef USE_EMU_INHERENT_SPEC
bool GUI::ShowConfigureDialog()
{
	bool rc = false;
	PostEtSystemPause(true);
	if (configbox->Show(hWindow) == IDOK) {
		save_config();
		ChangeLedBoxPosition(config.led_pos);
		rc = true;
	}
	PostEtSystemPause(false);
	SetFocusToMainWindow();
	return rc;
}

bool GUI::ShowKeybindDialog()
{
	bool rc = false;
	PostEtSystemPause(true);
	if (keybindbox->Show(hWindow) == IDOK) {
		emu->save_keybind();
		rc = true;
	}
	PostEtSystemPause(false);
	SetFocusToMainWindow();
	return rc;
}

/// create volume dialog
bool GUI::ShowVolumeDialog()
{
	volumebox->Show(hWindow);
	SetFocusToMainWindow();
	return true;
}

bool GUI::ShowSndFilterDialog()
{
#ifdef USE_DEBUG_SOUND_FILTER
	sndfilterbox->Show(hWindow);
#endif
	return true;
}
#endif

/// create about dialog
bool GUI::ShowAboutDialog()
{
	aboutbox->Show(hWindow);
	SetFocusToMainWindow();
	return true;
}

#ifdef USE_DATAREC
bool GUI::ShowLoadDataRecDialog()
{
	const _TCHAR *filter[] =
#if defined(DATAREC_PC8801)
		{ _("Supported Files (*.cas;*.cmt;*.t88)"), _("All Files (*.*)"), NULL };
#elif defined(DATAREC_BINARY_ONLY)
		{ _("Supported Files (*.cas;*.cmt)"), _("All Files (*.*)"), NULL };
#elif defined(DATAREC_TAP)
		{ _("Supported Files (*.wav;*.cas;*.tap)"), _("All Files (*.*)"), NULL };
#elif defined(DATAREC_MZT)
		{ _("Supported Files (*.wav;*.cas;*.mzt;*.m12)"), _("All Files (*.*)"), NULL };
#elif defined(USE_EMU_INHERENT_SPEC)
		{ _("Supported Files (*.l3;*.l3b;*.l3c;*.wav;*.t9x)"), _("All Files (*.*)"), NULL };
#else
		{ _("Supported Files (*.wav;*.cas)"),_("All Files (*.*)"), NULL };
#endif

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
		_("Play Data Recorder Tape"),
		config.initial_datarec_path,
		_T("l3"),
		false
	);
	if (rc) {
		PostEtLoadDataRecMessage(fbox.GetPath());
	} else {
		PostEtSystemPause(false);
	}
	return rc;
}

bool GUI::ShowSaveDataRecDialog()
{
	const _TCHAR *filter[] =
#if defined(DATAREC_PC8801)
		{ _("Supported Files (*.cas;*.cmt)"), _("All Files (*.*)"), NULL };
#elif defined(DATAREC_BINARY_ONLY)
		{ _("Supported Files (*.cas;*.cmt)"), _("All Files (*.*)"), NULL };
#elif defined(DATAREC_TAP)
		{ _("Supported Files (*.wav;*.cas)"), _("All Files (*.*)"), NULL };
#elif defined(DATAREC_MZT)
		{ _("Supported Files (*.wav;*.cas)"), _("All Files (*.*)"), NULL };
#elif defined(USE_EMU_INHERENT_SPEC)
		{ _("L3 File (*.l3)"), _("L3B File (*.l3b)"), _("L3C File (*.l3c)"), _("Wave File (*.wav)"), _("T9X File (*.t9x)"), _("All Files (*.*)"), NULL };
#else
		{ _("Supported Files (*.wav;*.cas)"), _("All Files (*.*)"), NULL };
#endif

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
		_("Record Data Recorder Tape"),
		config.initial_datarec_path,
		_T("l3"),
		true
	);
	if (rc) {
		PostEtSaveDataRecMessage(fbox.GetPath());
	} else {
		PostEtSystemPause(false);
	}
	return rc;
}

void GUI::set_datarec_file_menu(HMENU hMenu, UINT uItem)
{
	MENUITEMINFO minfo;
	_TCHAR str[_MAX_PATH];

	minfo.cbSize = sizeof(MENUITEMINFO);
	minfo.fMask = MIIM_TYPE;
	minfo.fType = MFT_STRING;

	if (emu->datarec_opened(true)) {
		_tcscpy(str,_("Play ["));
		_tcscat(str, UTILITY::trim_center(config.opened_datarec_path.path, TRIM_STRING_SIZE));
		_tcscat(str,_T("]"));
	} else {
		_tcscpy(str,_("Play..."));
	}
	minfo.dwTypeData = str;
	minfo.cch = (UINT)_tcslen(str);
	SetMenuItemInfo(hMenu, uItem, FALSE, &minfo);

	if (emu->datarec_opened(false)) {
		_tcscpy(str,_("Rec ["));
		_tcscat(str, UTILITY::trim_center(config.opened_datarec_path.path, TRIM_STRING_SIZE));
		_tcscat(str,_T("]"));
	} else {
		_tcscpy(str,_("Rec..."));
	}
	minfo.dwTypeData = str;
	minfo.cch = (UINT)_tcslen(str);
	SetMenuItemInfo(hMenu, uItem + 1, FALSE, &minfo);

//	CheckMenuRadioItem(hMenu, uItem, uItem + 1, uItem + opened_tape_type - 1, MF_BYCOMMAND);
	CheckMenuItem(hMenu, uItem, IsOpenedLoadDataRecFile() ? MF_CHECKED : MF_UNCHECKED);
	CheckMenuItem(hMenu, uItem + 1, IsOpenedSaveDataRecFile() ? MF_CHECKED : MF_UNCHECKED);
}
#endif

#ifdef USE_FD1
bool GUI::ShowOpenFloppyDiskDialog(int drv)
{
	const _TCHAR *filter[] =
#ifndef USE_EMU_INHERENT_SPEC
		{ _("Supported Files (*.d88;*.d77;*.td0;*.imd;*.dsk;*.fdi;*.hdm;*.tfd;*.xdf;*.2d;*.sf7)"), _("All Files (*.*)"), NULL };
#else
		{ _("Supported Files (*.d88;*.td0;*.imd;*.dsk;*.fdi;*.hdm;*.tfd;*.xdf;*.2d;*.sf7)"), _("All Files (*.*)"), NULL };
#endif

	DWORD flags;
	_TCHAR title[128];
	_stprintf(title, _("Open Floppy Disk %d"), drv);
	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
		title,
		config.initial_disk_path,
		_T("d88"),
		false
	);
	flags = fbox.GetFlags();
	if(rc) {
		PostEtOpenFloppyMessage(drv, fbox.GetPath(), 0, flags, true);
	} else {
		PostEtSystemPause(false);
	}
	return rc;
}

int GUI::ShowSelectFloppyDriveDialog(int drv)
{
//	PostEtSystemPause(true);
	SelDrvBox dlg(hInstance, font, emu);
	dlg.SetDrive(drv);
	dlg.SetPrefix(_("FDD"));
	int new_drv = (int)dlg.Show(hWindow);
	return new_drv >= 0 ? new_drv : drv;
}

void GUI::set_disk_file_menu(HMENU hMenu, UINT uItem, int drv)
{
	MENUITEMINFO minfo;
	_TCHAR str[_MAX_PATH];

	minfo.cbSize = sizeof(MENUITEMINFO);
	minfo.fMask = MIIM_TYPE;
	minfo.fType = MFT_STRING;

	bool ins = InsertedFloppyDisk(drv);

	if (ins) {
		_tcscpy(str,_("Open ["));
		_tcscat(str,UTILITY::trim_center(config.opened_disk_path[drv].path, TRIM_STRING_SIZE));
		if (config.opened_disk_path[drv].num > 0) {
			_stprintf(&str[_tcslen(str)], _T(" : %d"), config.opened_disk_path[drv].num + 1);
		}
		_tcscat(str,_T("]"));
	} else {
		_tcscpy(str,_("Open..."));
	}

	minfo.dwTypeData = str;
	minfo.cch = (UINT)_tcslen(str);

	SetMenuItemInfo(hMenu, uItem, FALSE, &minfo);
	CheckMenuItem(hMenu, uItem, ins ? MF_CHECKED : MF_UNCHECKED);
}

#ifdef USE_EMU_INHERENT_SPEC
void GUI::set_disk_side_menu(HMENU hMenu, UINT uItem, int drv)
{
	if (emu) {
		MENUITEMINFO minfo;
		_TCHAR str[40];

		minfo.cbSize = sizeof(MENUITEMINFO);
		minfo.fMask = MIIM_TYPE;
		minfo.fType = MFT_STRING;

		int side = GetSideFloppyDisk(drv);
		if (side == 1 && config.fdd_type == 1) {
			_tcscpy(str,_("Change Side to A"));
		} else {
			_tcscpy(str,_("Change Side to B"));
		}
		minfo.dwTypeData = str;
		minfo.cch = (UINT)_tcslen(str);

		SetMenuItemInfo(hMenu, uItem, FALSE, &minfo);
		EnableMenuItem(hMenu, uItem, side >= 0 && config.fdd_type == 1 ? MF_ENABLED : MF_GRAYED);
	}
}
#endif

#endif	// USE_FD1

#ifdef USE_AUTO_KEY
bool GUI::ShowOpenAutoKeyDialog()
{
	const _TCHAR *filter[] =
		{ _("Supported Files (*.txt;*.bas;*.lpt)"), _("All Files (*.*)"), NULL };

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
		_("Open Text File"),
		config.initial_autokey_path,
		_T("txt"),
		false
	);
	if(rc) {
		PostEtLoadAutoKeyMessage(fbox.GetPath());
	} else {
		PostEtSystemPause(false);
	}
	return rc;
}
bool GUI::StartAutoKey(void)
{
	return start_auto_key_from_clipboard(emu);
}
#endif

#ifdef USE_EMU_INHERENT_SPEC
bool GUI::ShowSaveStateDialog(bool cont)
{
	const _TCHAR *filter[] =
		{ _("Supported Files (*.l3r)"), _("All Files (*.*)"), NULL };

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	memset(config.saved_state_path, 0, sizeof(config.saved_state_path));
	bool rc = fbox.Show(
		filter,
		_("Save Status Data"),
		config.initial_state_path,
		_T("l3r"),
		true
	);
	if (rc) {
		PostEtSaveStatusMessage(fbox.GetPath(), cont);
	} else {
		PostEtSystemPause(false);
	}
	return rc;
}

bool GUI::ShowLoadStateDialog()
{
	const _TCHAR *filter[] =
		{ _("Supported Files (*.l3r)"), _("All Files (*.*)"), NULL };

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
		_("Load Status Data"),
		config.initial_state_path,
		_T("l3r"),
		false
		);
	if(rc) {
		PostEtLoadStatusMessage(fbox.GetPath());
	} else {
		PostEtSystemPause(false);
	}
	return rc;
}

#ifdef USE_KEY_RECORD
bool GUI::ShowPlayRecKeyDialog()
{
	const _TCHAR *filter[] =
		{ _("Supported Files (*.l3k)"), _("All Files (*.*)"), NULL };

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
		_("Play Recorded Keys"),
		config.initial_state_path,
		_T("l3k"),
		false
		);
	if(rc) {
		PostEtLoadRecKeyMessage(fbox.GetPath());
	} else {
		PostEtSystemPause(false);
	}
	return rc;
}

bool GUI::ShowRecordRecKeyDialog()
{
	const _TCHAR *filter[] =
		{ _("Supported Files (*.l3k)"), _("All Files (*.*)"), NULL };

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
		_("Record Input Keys"),
		config.initial_state_path,
		_T("l3k"),
		true
		);
	if(rc) {
		PostEtSaveRecKeyMessage(fbox.GetPath(), false);
	} else {
		PostEtSystemPause(false);
	}
	return rc;
}
bool GUI::ShowRecordStateAndRecKeyDialog()
{
	bool rc = ShowSaveStateDialog(true);
	if (!rc) return rc;
	return ShowRecordRecKeyDialog();
}
#endif
#endif	// USE_EMU_INHERENT_SPEC

#ifdef USE_PRINTER
bool GUI::ShowSavePrinterDialog(int drv)
{
	const _TCHAR *filter[] =
		{ _("Supported Files (*.lpt)"), _("All Files (*.*)"), NULL };

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
		_("Save Printing Data"),
		config.initial_printer_path,
		_T("lpt"),
		true
	);
	if (rc) {
		PostEtSavePrinterMessage(drv, fbox.GetPath());
	} else {
		PostEtSystemPause(false);
	}
	return rc;
}
#endif

#ifdef USE_CART
void open_cart_dialog()
{
	const _TCHAR *filter[] =
#ifdef _X1TWIN
		{ _("Supported Files (*.pce)"), _("All Files (*.*)"), NULL };
#else
		{ _("Supported Files (*.rom;*.bin)"), _("All Files (*.*)"), NULL };
#endif

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
#ifdef _X1TWIN
		_("HuCARD"),
#else
		_("Game Cartridge"),
#endif
		config.initial_cart_path
	);
	if(rc) {
		UPDATE_HISTORY(path, config.recent_cart_path);
		emu->open_cart(fbox.GetPath());
	} else {
		PostEtSystemPause(false);
	}
}
#endif

#ifdef USE_QUICKDISK
void open_quickdisk_dialog()
{
	const _TCHAR *filter[] =
		{ _("Supported Files (*.mzt)"), _("All Files (*.*)"), NULL };

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
		_("Quick Disk"),
		config.initial_quickdisk_path
	);
	if(rc) {
		UPDATE_HISTORY(path, config.recent_quickdisk_path);
		emu->open_quickdisk(fbox.GetPath());
	} else {
		PostEtSystemPause(false);
	}
}
#endif

#ifdef USE_MEDIA
void open_media_dialog()
{
	const _TCHAR *filter[] =
		{ _("Supported Files (*.m3u)"), _("All Files (*.*)"), NULL };

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
		_("Sound Cassette Tape"),
		config.initial_media_path
	);
	if(rc) {
		UPDATE_HISTORY(path, config.recent_media_path);
		emu->open_media(fbox.GetPath());
	} else {
		PostEtSystemPause(false);
	}
}
#endif

#ifdef USE_BINARY_FILE1
void open_binary_dialog(int drv, bool load)
{
	const _TCHAR *filter[] =
		{ _("Supported Files (*.ram;*.bin)"), _("All Files (*.*)"), NULL };

	FileBox fbox(hWindow);

	PostEtSystemPause(true);
	bool rc = fbox.Show(
		filter,
#if defined(_PASOPIA) || defined(_PASOPIA7)
		_("RAM Pack Cartridge"),
#else
		_("Memory Dump"),
#endif
		config.initial_binary_path
	);
	if(rc) {
		UPDATE_HISTORY(path, config.recent_binary_path[drv]);
		if(load) {
			emu->load_binary(drv, fbox.GetPath());
		}
		else {
			emu->save_binary(drv, fbox.GetPath());
		}
	}
	PostEtSystemPause(false);
}
#endif

/**
 *	open dropped file
 */
bool GUI::OpenDroppedFile(void *param)
{
	_TCHAR dropped_file[_MAX_PATH];

	HDROP hDrop=(HDROP)param;
	// get first filename
	DragQueryFile(hDrop, 0, dropped_file, _MAX_PATH);
    DragFinish(hDrop);
	if (_tcslen(dropped_file) == 0) {
		return false;
	}

	return OpenFileByExtention(dropped_file);
}

//
//
//
//
//
void GUI::update_menu(HMENU hMenu, int pos)
{
	if (hMenu != GetSubMenu(GetMenu(hWindow), pos)) return;
	_TCHAR buf[_MAX_PATH];

#ifdef MENU_POS_CONTROL
	if(pos == MENU_POS_CONTROL) {
		// control menu
#ifdef USE_DIPSWITCH
		for(int i = 0; i < 8; i++) {
#ifdef USE_EMU_INHERENT_SPEC
			CheckMenuItem(hMenu, ID_DIPSWITCH1 + i, (GetDipswitch() & (1 << i)) ? MF_CHECKED : MF_UNCHECKED);
#else
			CheckMenuItem(hMenu, ID_DIPSWITCH1 + i, !(GetDipswitch() & (1 << i)) ? MF_CHECKED : MF_UNCHECKED);
#endif
		}
#endif
#ifdef USE_EMU_INHERENT_SPEC
		CheckMenuRadioItem(hMenu, ID_FDD_TYPE_1, ID_FDD_TYPE_3, ID_FDD_TYPE_1 + NextFddType(), MF_BYCOMMAND);
#endif
#ifdef _HC80
		if(config.device_type >= 0 && config.device_type < 3) {
			CheckMenuRadioItem(hMenu, ID_HC80_RAMDISK0, ID_HC80_RAMDISK2, ID_HC80_RAMDISK0 + config.device_type, MF_BYCOMMAND);
		}
#endif
#ifdef _MZ800
		if(config.boot_mode >= 0 && config.boot_mode < 2) {
			CheckMenuRadioItem(hMenu, ID_MZ800_MODE_MZ800, ID_MZ800_MODE_MZ700, ID_MZ800_MODE_MZ800 + config.boot_mode, MF_BYCOMMAND);
		}
#endif
#ifdef _PASOPIA
		if(config.boot_mode >= 0 && config.boot_mode < 5) {
			CheckMenuRadioItem(hMenu, ID_PASOPIA_MODE_TBASIC_V1_0, ID_PASOPIA_MODE_MINI_PASCAL, ID_PASOPIA_MODE_TBASIC_V1_0 + config.boot_mode, MF_BYCOMMAND);
		}
		if(config.device_type >= 0 && config.boot_mode < 3) {
			CheckMenuRadioItem(hMenu, ID_PASOPIA_DEVICE_RAM_PAC, ID_PASOPIA_DEVICE_JOYSTICK, ID_PASOPIA_DEVICE_RAM_PAC + config.device_type, MF_BYCOMMAND);
		}
#endif
#ifdef _PC98DO
		if(config.boot_mode >= 0 && config.boot_mode < 5) {
			CheckMenuRadioItem(hMenu, ID_PC98DO_MODE_PC98, ID_PC8801_MODE_N, ID_PC98DO_MODE_PC98 + config.boot_mode, MF_BYCOMMAND);
		}
#endif
#ifdef _PC8801MA
		if(config.boot_mode >= 0 && config.boot_mode < 4) {
			CheckMenuRadioItem(hMenu, ID_PC8801_MODE_V1S, ID_PC8801_MODE_N, ID_PC8801_MODE_V1S + config.boot_mode, MF_BYCOMMAND);
		}
		if(config.device_type >= 0 && config.boot_mode < 3) {
//			CheckMenuRadioItem(hMenu, ID_PC8801_DEVICE_JOYSTICK, ID_PC8801_DEVICE_JOYMOUSE, ID_PC8801_DEVICE_JOYSTICK + config.boot_mode, MF_BYCOMMAND);
			// joymouse is not supported yet...
			CheckMenuRadioItem(hMenu, ID_PC8801_DEVICE_JOYSTICK, ID_PC8801_DEVICE_MOUSE, ID_PC8801_DEVICE_JOYSTICK + config.device_type, MF_BYCOMMAND);
		}
#endif
#if defined(_PC9801E) || defined(_PC9801VM) || defined(_PC98DO) || defined(_PC8801MA)
		if(config.cpu_clock_low) {
			CheckMenuRadioItem(hMenu, ID_PC9801_CPU_CLOCK_HIGH, ID_PC9801_CPU_CLOCK_LOW, ID_PC9801_CPU_CLOCK_LOW, MF_BYCOMMAND);
		}
		else {
			CheckMenuRadioItem(hMenu, ID_PC9801_CPU_CLOCK_HIGH, ID_PC9801_CPU_CLOCK_LOW, ID_PC9801_CPU_CLOCK_HIGH, MF_BYCOMMAND);
		}
#endif
		if(config.cpu_power >= 0 && config.cpu_power <= 5) {
			CheckMenuRadioItem(hMenu, ID_CPU_POWER0, ID_CPU_POWER5, ID_CPU_POWER0 + GetCPUPower(), MF_BYCOMMAND);
		}
#ifdef USE_EMU_INHERENT_SPEC
		CheckMenuItem(hMenu, ID_PAUSE,    NowPause() ? MF_CHECKED : MF_UNCHECKED);
		CheckMenuItem(hMenu, ID_SYNC_IRQ, NowSyncIRQ() ? MF_CHECKED : MF_UNCHECKED);
#endif
#ifdef USE_AUTO_KEY
		// auto key
		bool now_paste = true;
		if(emu) {
			now_paste = IsRunningAutoKey();
		}
		EnableMenuItem(hMenu, ID_AUTOKEY_OPEN, now_paste ? MF_GRAYED : MF_ENABLED);
		EnableMenuItem(hMenu, ID_AUTOKEY_START, now_paste ? MF_GRAYED : MF_ENABLED);
#endif
#ifdef USE_EMU_INHERENT_SPEC
#ifdef USE_KEY_RECORD
		CheckMenuItem(hMenu, ID_RECKEY_PLAY, NowPlayingRecKey() ? MF_CHECKED : MF_UNCHECKED);
		CheckMenuItem(hMenu, ID_RECKEY_REC,  NowRecordingRecKey() ? MF_CHECKED : MF_UNCHECKED);
#endif
#endif
	}
#endif
#ifdef MENU_POS_CART
	else if(pos == MENU_POS_CART) {
		// cartridge
		UPDATE_RECENT_MENU(ID_RECENT_CART, config.recent_cart_path);
	}
#endif
#define UPDATE_MENU_FD(drv, ID_OPEN_FD, ID_CHANGE_FD, ID_WRITEPROTECT_FD, ID_RECENT_FD, ID_D88_FILE_PATH, ID_SELECT_D88_BANK) { \
	UPDATE_MULTIVOLUME_MENU(drv, ID_D88_FILE_PATH, ID_SELECT_D88_BANK); \
	UPDATE_RECENT_MENU(ID_RECENT_FD, config.recent_disk_path[drv]); \
	set_disk_side_menu(hMenu, ID_CHANGE_FD, drv); \
	set_disk_file_menu(hMenu, ID_OPEN_FD, drv); \
	EnableMenuItem(hMenu, ID_WRITEPROTECT_FD, InsertedFloppyDisk(drv) ? MF_ENABLED : MF_GRAYED); \
	CheckMenuItem(hMenu, ID_WRITEPROTECT_FD, WriteProtectedFloppyDisk(drv) ? MF_CHECKED : MF_UNCHECKED); \
}
#ifdef MENU_POS_FD1
	else if(pos == MENU_POS_FD1) {
		// floppy drive #1
		UPDATE_MENU_FD(0, ID_OPEN_FD1, ID_CHANGE_FD1, ID_WRITEPROTECT_FD1, ID_RECENT_FD1, ID_D88_FILE_PATH1, ID_SELECT_D88_BANK1)
	}
#endif
#ifdef MENU_POS_FD2
	else if(pos == MENU_POS_FD2) {
		// floppy drive #2
		UPDATE_MENU_FD(1, ID_OPEN_FD2, ID_CHANGE_FD2, ID_WRITEPROTECT_FD2, ID_RECENT_FD2, ID_D88_FILE_PATH2, ID_SELECT_D88_BANK2)
	}
#endif
#ifdef MENU_POS_FD3
	else if(pos == MENU_POS_FD3) {
		// floppy drive #3
		UPDATE_MENU_FD(2, ID_OPEN_FD3, ID_CHANGE_FD3, ID_WRITEPROTECT_FD3, ID_RECENT_FD3, ID_D88_FILE_PATH3, ID_SELECT_D88_BANK3)
	}
#endif
#ifdef MENU_POS_FD4
	else if(pos == MENU_POS_FD4) {
		// floppy drive #4
		UPDATE_MENU_FD(3, ID_OPEN_FD4, ID_CHANGE_FD4, ID_WRITEPROTECT_FD4, ID_RECENT_FD4, ID_D88_FILE_PATH4, ID_SELECT_D88_BANK4)
	}
#endif
#ifdef MENU_POS_FD5
	else if(pos == MENU_POS_FD5) {
		// floppy drive #5
		UPDATE_MENU_FD(4, ID_OPEN_FD5, ID_CHANGE_FD5, ID_WRITEPROTECT_FD5, ID_RECENT_FD5, ID_D88_FILE_PATH5, ID_SELECT_D88_BANK5)
	}
#endif
#ifdef MENU_POS_FD6
	else if(pos == MENU_POS_FD6) {
		// floppy drive #6
		UPDATE_MENU_FD(5, ID_OPEN_FD6, ID_CHANGE_FD6, ID_WRITEPROTECT_FD6, ID_RECENT_FD6, ID_D88_FILE_PATH6, ID_SELECT_D88_BANK6)
	}
#endif
#ifdef MENU_POS_QUICKDISK
	else if(pos == MENU_POS_QUICKDISK) {
		// quick disk drive
		UPDATE_RECENT_MENU(ID_RECENT_QUICKDISK, config.recent_quickdisk_path);
	}
#endif
#ifdef MENU_POS_DATAREC
	else if(pos == MENU_POS_DATAREC) {
		// data recorder
		UPDATE_RECENT_MENU(ID_RECENT_DATAREC, config.recent_datarec_path);

		set_datarec_file_menu(hMenu, ID_PLAY_DATAREC);
		// realmode
		CheckMenuItem(hMenu, ID_REAL_DATAREC, NowRealModeDataRec() ? MF_CHECKED : MF_UNCHECKED);
	}
#endif
#ifdef MENU_POS_MEDIA
	else if(pos == MENU_POS_MEDIA) {
		// media
		UPDATE_RECENT_MENU(ID_RECENT_MEDIA, config.recent_media_path);
	}
#endif
#ifdef MENU_POS_BINARY1
	else if(pos == MENU_POS_BINARY1) {
		// binary #1
		#define UPDATE_MENU_BINARY(drv, ID_RECENT_BINARY) \
			UPDATE_RECENT_MENU(ID_RECENT_BINARY, config.recent_binary_path[drv])
		UPDATE_MENU_BINARY(0, ID_RECENT_BINARY1)
	}
#endif
#ifdef MENU_POS_BINARY2
	else if(pos == MENU_POS_BINARY2) {
		// binary #2
		UPDATE_MENU_BINARY(1, ID_RECENT_BINARY2)
	}
#endif
#ifdef MENU_POS_SCREEN
	else if(pos == MENU_POS_SCREEN) {
		// recording
		bool now_rec = true;
		if(emu) {
			now_rec = NowRecordingVideo();
		}
#ifdef USE_EMU_INHERENT_SPEC
		// rec video size
		InsertMenu(hMenu, ID_SCREEN_REC_SIZE1, MF_BYCOMMAND | MF_STRING, ID_SCREEN_BOTTOM, _T(""));
		for(int i = 0; i <= (ID_SCREEN_REC_SIZE2 - ID_SCREEN_REC_SIZE1); i++) {
			DeleteMenu(hMenu, ID_SCREEN_REC_SIZE1 + i, MF_BYCOMMAND);
		}
		for(int i = 0; i <= (ID_SCREEN_REC_SIZE2 - ID_SCREEN_REC_SIZE1); i++) {
			GetRecordVideoSizeStr(i, buf);
			InsertMenu(hMenu, ID_SCREEN_BOTTOM, MF_BYCOMMAND | MF_STRING, ID_SCREEN_REC_SIZE1 + i, buf);
		}
		DeleteMenu(hMenu, ID_SCREEN_BOTTOM, MF_BYCOMMAND);
		CheckMenuRadioItem(hMenu, ID_SCREEN_REC_SIZE1, ID_SCREEN_REC_SIZE2, ID_SCREEN_REC_SIZE1 + config.screen_video_size, MF_BYCOMMAND);
		for(UINT i = ID_SCREEN_REC_SIZE1; i<= ID_SCREEN_REC_SIZE2; i++) {
			EnableMenuItem(hMenu, i, now_rec ? MF_GRAYED : MF_ENABLED);
		}
#endif
		for(UINT i = ID_SCREEN_REC60; i<= ID_SCREEN_REC10; i++) {
			EnableMenuItem(hMenu, i, now_rec ? MF_GRAYED : MF_ENABLED);
		}
		if (now_rec) {
			CheckMenuRadioItem(hMenu, ID_SCREEN_REC60, ID_SCREEN_REC10, ID_SCREEN_REC60 + rec_fps_no, MF_BYCOMMAND);
		} else {
			CheckMenuRadioItem(hMenu, ID_SCREEN_REC60, ID_SCREEN_REC10, ID_SCREEN_REC60 - 1, MF_BYCOMMAND);
		}
//		EnableMenuItem(hMenu, ID_SCREEN_STOP, now_stop ? MF_GRAYED : MF_ENABLED);

#ifdef USE_EMU_INHERENT_SPEC
		// frame rate
		CheckMenuRadioItem(hMenu, ID_SCREEN_VFRAME, ID_SCREEN_FPS10, ID_SCREEN_VFRAME + config.fps_no + 1, MF_BYCOMMAND);
//		for(UINT i = ID_SCREEN_VFRAME; i<= ID_SCREEN_FPS10; i++) {
//			EnableMenuItem(hMenu, i, now_rec ? MF_GRAYED : MF_ENABLED);
//		}
#endif

		// screen mode
		UINT last = ID_SCREEN_WINDOW1;
		InsertMenu(hMenu, ID_SCREEN_WINDOW1, MF_BYCOMMAND | MF_STRING, ID_SCREEN_BOTTOM, _T(""));
		for(int i = 0; i < WINDOW_MODE_MAX; i++) {
			DeleteMenu(hMenu, ID_SCREEN_WINDOW1 + i, MF_BYCOMMAND);
		}
		for(int i = 0; i < GetWindowModeCount(); i++) {
			GetWindowModeStr(i, buf);
			InsertMenu(hMenu, ID_SCREEN_BOTTOM, MF_BYCOMMAND | MF_STRING, ID_SCREEN_WINDOW1 + i, buf);
			last = ID_SCREEN_WINDOW1 + i;
		}
		DeleteMenu(hMenu, ID_SCREEN_BOTTOM, MF_BYCOMMAND);

		InsertMenu(hMenu, ID_SCREEN_FULLSCREEN1, MF_BYCOMMAND | MF_STRING, ID_SCREEN_BOTTOM, _T(""));
		for(int i = 0; i < SCREEN_MODE_MAX; i++) {
			DeleteMenu(hMenu, ID_SCREEN_FULLSCREEN1 + i, MF_BYCOMMAND);
		}
		for(int i = 0; i < GetFullScreenModeCount(); i++) {
			MENUITEMINFO info;
			ZeroMemory(&info, sizeof(info));
			info.cbSize = sizeof(info);
			GetFullScreenModeStr(i, buf);
			info.fMask = MIIM_TYPE | MIIM_ID;
			info.fType = MFT_STRING;
			info.wID = ID_SCREEN_FULLSCREEN1 + i;
			info.dwTypeData = buf;
			InsertMenuItem(hMenu, ID_SCREEN_BOTTOM, FALSE, &info);
			EnableMenuItem(hMenu, ID_SCREEN_FULLSCREEN1 + i, IsFullScreen() ? MF_GRAYED : MF_ENABLED);
			last = ID_SCREEN_FULLSCREEN1 + i;
		}
		DeleteMenu(hMenu, ID_SCREEN_BOTTOM, MF_BYCOMMAND);

		if(config.window_mode >= 0 && config.window_mode < GetWindowModeCount()) {
			CheckMenuRadioItem(hMenu, ID_SCREEN_WINDOW1, last, ID_SCREEN_WINDOW1 + config.window_mode, MF_BYCOMMAND);
		}
		else if(config.window_mode >= 8 && config.window_mode < GetFullScreenModeCount() + 8) {
			CheckMenuRadioItem(hMenu, ID_SCREEN_FULLSCREEN1, last, ID_SCREEN_FULLSCREEN1 + config.window_mode - 8, MF_BYCOMMAND);
		}
		// stretch screen
		CheckMenuItem(hMenu, ID_SCREEN_STRETCH, GetStretchScreen() == 1 ? MF_CHECKED : MF_UNCHECKED);
		CheckMenuItem(hMenu, ID_SCREEN_CUTOUT, GetStretchScreen() == 2 ? MF_CHECKED : MF_UNCHECKED);
		// pixel aspect ratio
		last = GetPixelAspectModeCount();
		InsertMenu(hMenu, ID_SCREEN_PIXEL_ASPECT0, MF_BYCOMMAND | MF_STRING, ID_SCREEN_BOTTOM, _T(""));
		for(UINT i = 0; i < last; i++) {
			DeleteMenu(hMenu, ID_SCREEN_PIXEL_ASPECT0 + i, MF_BYCOMMAND);
		}
		for(UINT i = 0; i < last; i++) {
			GetPixelAspectModeStr(i, buf);
			InsertMenuA(hMenu, ID_SCREEN_BOTTOM, MF_BYCOMMAND | MF_STRING, ID_SCREEN_PIXEL_ASPECT0 + i, buf);
		}
		DeleteMenu(hMenu, ID_SCREEN_BOTTOM, MF_BYCOMMAND);
		CheckMenuRadioItem(hMenu, ID_SCREEN_PIXEL_ASPECT0, ID_SCREEN_PIXEL_ASPECT0 + last - 1, ID_SCREEN_PIXEL_ASPECT0 + GetPixelAspectMode(), MF_BYCOMMAND);

#ifdef USE_MONITOR_TYPE
		if(config.monitor_type >= 0 && config.monitor_type < USE_MONITOR_TYPE) {
			CheckMenuRadioItem(hMenu, ID_SCREEN_MONITOR_TYPE0, ID_SCREEN_MONITOR_TYPE0 + USE_MONITOR_TYPE - 1, ID_SCREEN_MONITOR_TYPE0 + config.monitor_type, MF_BYCOMMAND);
		}
#elif defined(USE_SCREEN_ROTATE)
		if(config.monitor_type >= 0 && config.monitor_type < 2) {
			CheckMenuRadioItem(hMenu, ID_SCREEN_MONITOR_TYPE0, ID_SCREEN_MONITOR_TYPE1, ID_SCREEN_MONITOR_TYPE0 + config.monitor_type, MF_BYCOMMAND);
		}
#endif
#ifdef USE_SCANLINE
		// scanline
		CheckMenuRadioItem(hMenu, ID_SCREEN_SCANLINE0, ID_SCREEN_SCANLINE3, ID_SCREEN_SCANLINE0 + NowScanLine(), MF_BYCOMMAND);
#endif
#ifdef USE_EMU_INHERENT_SPEC
		CheckMenuItem(hMenu, ID_SCREEN_AFTERIMAGE1, (GetAfterImageMode() & 1) ? MF_CHECKED : MF_UNCHECKED);
		CheckMenuItem(hMenu, ID_SCREEN_AFTERIMAGE2, (GetAfterImageMode() & 2) ? MF_CHECKED : MF_UNCHECKED);
		CheckMenuItem(hMenu, ID_SCREEN_KEEPIMAGE1, (GetKeepImageMode() & 1) ? MF_CHECKED : MF_UNCHECKED);
		CheckMenuItem(hMenu, ID_SCREEN_KEEPIMAGE2, (GetKeepImageMode() & 2) ? MF_CHECKED : MF_UNCHECKED);
#ifdef USE_DIRECT3D
		CheckMenuItem(hMenu, ID_SCREEN_D3D_SYNC, (config.use_direct3d & 1) ? MF_CHECKED : MF_UNCHECKED);
		CheckMenuItem(hMenu, ID_SCREEN_D3D_ASYNC, (config.use_direct3d & 2) ? MF_CHECKED : MF_UNCHECKED);
		CheckMenuRadioItem(hMenu, ID_SCREEN_D3D_FILTER0, ID_SCREEN_D3D_FILTER2, ID_SCREEN_D3D_FILTER0 + GetDirect3DFilter(), MF_BYCOMMAND);
		bool enable_d3d = emu->enabled_direct3d();
		EnableMenuItem(hMenu, ID_SCREEN_D3D_SYNC, enable_d3d ? MF_ENABLED : MF_GRAYED);
		EnableMenuItem(hMenu, ID_SCREEN_D3D_ASYNC, enable_d3d ? MF_ENABLED : MF_GRAYED);
		EnableMenuItem(hMenu, ID_SCREEN_D3D_FILTER0, enable_d3d ? MF_ENABLED : MF_GRAYED);
		EnableMenuItem(hMenu, ID_SCREEN_D3D_FILTER1, enable_d3d ? MF_ENABLED : MF_GRAYED);
		EnableMenuItem(hMenu, ID_SCREEN_D3D_FILTER2, enable_d3d ? MF_ENABLED : MF_GRAYED);
#else
		CheckMenuItem(hMenu, ID_SCREEN_D3D_SYNC, MF_UNCHECKED);
		CheckMenuItem(hMenu, ID_SCREEN_D3D_ASYNC, MF_UNCHECKED);
		CheckMenuItem(hMenu, ID_SCREEN_D3D_FILTER, MF_UNCHECKED);
		EnableMenuItem(hMenu, ID_SCREEN_D3D_SYNC, MF_GRAYED);
		EnableMenuItem(hMenu, ID_SCREEN_D3D_ASYNC, MF_GRAYED);
		EnableMenuItem(hMenu, ID_SCREEN_D3D_FILTER, MF_GRAYED);
#endif
#ifdef USE_D3D
		CheckMenuItem(hMenu, ID_SCREEN_WAIT_VSYNC, config.wait_vsync ? MF_CHECKED : MF_UNCHECKED);
#endif
#endif
	}
#endif
#ifdef MENU_POS_SOUND
	else if(pos == MENU_POS_SOUND) {
		// sound menu
		bool now_rec = false;
		if(emu) {
			now_rec = NowRecordingSound();
		}
		EnableMenuItem(hMenu, ID_SOUND_REC, now_rec ? MF_GRAYED : MF_ENABLED);
		CheckMenuItem(hMenu, ID_SOUND_REC, now_rec ? MF_CHECKED : MF_UNCHECKED);
//		EnableMenuItem(hMenu, ID_SOUND_STOP, now_stop ? MF_GRAYED : MF_ENABLED);

		CheckMenuRadioItem(hMenu, ID_SOUND_FREQ0, ID_SOUND_FREQ7, ID_SOUND_FREQ0 + GetSoundFrequencyNum(), MF_BYCOMMAND);
		CheckMenuRadioItem(hMenu, ID_SOUND_LATE0, ID_SOUND_LATE4, ID_SOUND_LATE0 + GetSoundLatencyNum(), MF_BYCOMMAND);
#ifdef USE_SOUND_DEVICE_TYPE
		if(config.sound_device_type >= 0 && config.sound_device_type < USE_SOUND_DEVICE_TYPE) {
			CheckMenuRadioItem(hMenu, ID_SOUND_DEVICE_TYPE0, ID_SOUND_DEVICE_TYPE0 + USE_SOUND_DEVICE_TYPE - 1, ID_SOUND_DEVICE_TYPE0 + config.sound_device_type, MF_BYCOMMAND);
		}
#endif
	}
#endif
#ifdef MENU_POS_DEVICE
	else if(pos == MENU_POS_DEVICE) {
		for(int i=0; i<MAX_PRINTER; i++) {
			int j = i * 4;
			int printer_buffer_size = 0;
			// printer menu
			if(emu) {
				printer_buffer_size = GetPrinterBufferSize(i);
			}
			EnableMenuItem(hMenu, ID_PRINTER0_SAVE + j,  printer_buffer_size <= 0 ? MF_GRAYED : MF_ENABLED);
			EnableMenuItem(hMenu, ID_PRINTER0_PRINT + j, printer_buffer_size <= 0 ? MF_GRAYED : MF_ENABLED);
//			EnableMenuItem(hMenu, ID_PRINTER0_CLEAR + j, printer_buffer_size <= 0 ? MF_GRAYED : MF_ENABLED);
			CheckMenuItem(hMenu, ID_PRINTER0_DIRECT + j, IsEnablePrinterDirect(i) ? MF_CHECKED : MF_UNCHECKED);
		}
		for(int i=0; i<MAX_COMM; i++) {
			int j = i * 2;
			// comm menu
			CheckMenuItem(hMenu, ID_COMM0_SERVER + j, IsEnableCommServer(i) ? MF_CHECKED : MF_UNCHECKED);
			CheckMenuItem(hMenu, ID_COMM0_CONNECT + j, NowConnectingComm(i) ? MF_CHECKED : MF_UNCHECKED);
		}
	}
#endif
#ifdef MENU_POS_OPTIONS
	else if(pos == MENU_POS_OPTIONS) {
		// options menu
		if (emu) {
			CheckMenuItem(hMenu, ID_OPTIONS_LEDBOX_SHOW, IsShownLedBox() ? MF_CHECKED : MF_UNCHECKED);
			CheckMenuItem(hMenu, ID_OPTIONS_LEDBOX_INSIDE, IsInsidedLedBox() ? MF_CHECKED : MF_UNCHECKED);
			CheckMenuItem(hMenu, ID_OPTIONS_MSGBOARD, IsShownMessageBoard() ? MF_CHECKED : MF_UNCHECKED);
			CheckMenuItem(hMenu, ID_OPTIONS_JOYPAD0, IsEnableJoypad(1) ? MF_CHECKED : MF_UNCHECKED);
			CheckMenuItem(hMenu, ID_OPTIONS_LIGHTPEN, IsEnableLightpen() ? MF_CHECKED : MF_UNCHECKED);
		}
	}
#endif
	DrawMenuBar(hWindow);
}

/// translate text string on the menu
void GUI::translate_menu(HMENU hMenu, int depth)
{
	_TCHAR buf[_MAX_PATH];
	MENUITEMINFO mii;

	if (depth >= 10) return;

	int nums = GetMenuItemCount(hMenu);
	for(int i=0; i<nums; i++) {
		memset(&mii, 0, sizeof(mii));
		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_SUBMENU | MIIM_STRING;
		mii.dwTypeData = buf;
		mii.cch = _MAX_PATH - 1;
		GetMenuItemInfo(hMenu, i, TRUE, &mii);
		if (mii.cch > 0) {
			mii.fMask = MIIM_STRING;
			_tcscpy(buf, _tgettext(mii.dwTypeData));
			mii.dwTypeData = buf;
			mii.cch = (UINT)_tcslen(buf);
			SetMenuItemInfo(hMenu, i, TRUE, &mii);
		}
		if (mii.hSubMenu != NULL) {
			translate_menu(mii.hSubMenu, depth + 1);
		}
	}
}

#endif /* GUI_TYPE_WINDOWS */

bool start_auto_key_from_clipboard(EMU *emu)
{
	bool rc = true;
	if(OpenClipboard(NULL)) {
		HANDLE hClip = GetClipboardData(CF_TEXT);
		if(hClip) {
			char *str = (char*)GlobalLock(hClip);
			emu->start_auto_key(str);
		}
		CloseClipboard();
	} else {
		rc = false;
	}
	return rc;
}

}; /* namespace GUI_WIN */
