/** @file floppy.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.11.05 -

	@brief [ floppy drive ]
*/

#ifndef _FLOPPY_H_
#define _FLOPPY_H_

#include "../vm.h"
#include "../../emu.h"
#include "../device.h"
#include "parsewav.h"

/* #define _DEBUG_FLOPPY */

#define SIG_FLOPPY_ACCESS			0
#define SIG_FLOPPY_READ_ID			1
//#define SIG_FLOPPY_READ_ID_TRACK_NUM		2
//#define SIG_FLOPPY_READ_ID_HEAD_NUM		3
#define SIG_FLOPPY_READ				6
#define SIG_FLOPPY_READ_TRACK		7
#define SIG_FLOPPY_WRITE			8
#define SIG_FLOPPY_WRITE_TRACK		9
#define SIG_FLOPPY_WRITEDELETE		10
#define SIG_FLOPPY_WRITEPROTECT		11
#define SIG_FLOPPY_STEP				12	// step and dirc
#define SIG_FLOPPY_HEADLOAD			13
#define SIG_FLOPPY_READY			14
#define SIG_FLOPPY_TRACK0			15
#define SIG_FLOPPY_INDEX			16
#define SIG_FLOPPY_DELETED			17

#define SIG_FLOPPY_CURRENTTRACK		18
#define SIG_FLOPPY_SECTOR_NUM		19
#define SIG_FLOPPY_SECTOR_SIZE		20
#define SIG_FLOPPY_TRACK_SIZE		21

// for 5inch mini floppy
#define SIG_FLOPPY_IRQ				26
#define SIG_FLOPPY_DRQ				27

#define FLOPPY_MAX_EVENT 20

//class FDC;
class DISK;

/**
	@brief floppy drive
*/
class FLOPPY : public DEVICE
{
private:
	DEVICE *d_fdc3, *d_fdc5;

	// config
	bool ignore_crc;

	// event
	int register_id[FLOPPY_MAX_EVENT];

	//
	uint8 drv_num;	// drive number
	uint8 drvsel;	// drive select + motor

	uint8 sidereg;	// head select

	uint8 index_hole;	// index hole
	uint8 head_load;	// head loaded

	uint8 density;	// density (double = 1)

	int sectorcnt;

	uint8 fdd5outreg;
	bool irqflg;
	bool irqflgprev;
	bool drqflg;

	// output signals
	outputs_t outputs_irq;
	outputs_t outputs_drq;

	// drive info
	typedef struct {
		int side;
		int track;
		int index;
		bool access;
		uint8 ready;	// ready warmup:01 on:11
		uint8 motor_warmup;	// motor warming up:1
		uint8 head_loading;
	} fdc_t;
	fdc_t fdc[MAX_DRIVE];

	// diskette info
	DISK* disk[MAX_DRIVE];

	int sample_rate;
	int max_vol;
	int wav_volume;
	int wav_enable;
	bool wav_loaded_at_first;

	// sound data 1st dimension [0]:3inch [1]:5inch  2nd [0]:seek sound [1]:motor sound
	_TCHAR wav_file[2][2][16];
	uint8 *wav_data[2][2];
	int    wav_size[2][2];
	// [0]:seek sound [1]:motor sound
	int    wav_play_pos[2];
	int    wav_play[2];

	//for resume
#pragma pack(1)
	struct vm_state_st_v1 {
		int register_id[6];
		uint8 drv_num;
		uint8 drvsel;
		uint8 sidereg;
		uint8 fdd5outreg;
		uint8 irqflg;
		uint8 drqflg;

		char reserved[2];
	};
	struct vm_state_st_v2 {
		int register_id[22];
		uint8 drv_num;
		uint8 drvsel;
		uint8 sidereg;
		uint8 fdd5outreg;
		uint8 irqflg;
		uint8 drqflg;

		char reserved[2];
	};
#pragma pack()
	// cannot write when load state from file
	bool ignore_write;

	inline void cancel_my_event(int);
	inline void register_my_event(int, int);
	void cancel_my_events();

	void set_drive_select(uint8 data);

	// irq/dma
	void set_irq(bool val);
	void set_drq(bool val);

	void load_wav();

	void motor(int drv, bool val);
public:
	FLOPPY(VM* parent_vm, EMU* parent_emu, const char* identifier) : DEVICE(parent_vm, parent_emu, identifier) {
		set_class_name("FLOPPY");
		init_output_signals(&outputs_irq);
		init_output_signals(&outputs_drq);
		d_fdc3 = NULL;
		d_fdc5 = NULL;
	}
	~FLOPPY() {}

	// common functions
	void initialize();
	void reset();
	void release();
	void write_io8(uint32 addr, uint32 data);
	uint32 read_io8(uint32 addr);
	void write_signal(int id, uint32 data, uint32 mask);
	uint32 read_signal(int id);

	// unique functions
	void set_context_fdc(DEVICE* device3, DEVICE* device5) {
		d_fdc3 = device3;
		d_fdc5 = device5;
	}
	void set_context_irq(DEVICE* device, int id, uint32 mask) {
		register_output_signal(&outputs_irq, device, id, mask);
	}
	void set_context_drq(DEVICE* device, int id, uint32 mask) {
		register_output_signal(&outputs_drq, device, id, mask);
	}

	// event callback
	void event_frame();
	void event_callback(int event_id, int err);

	bool search_track();
	bool verify_track();
	int  search_sector();
	int  search_sector(int sect);
	int  search_sector(int sect, bool compare_side, int side);
	bool make_track();
	bool parse_track();

	// user interface
	bool open_disk(int drv, const _TCHAR *path, int offset, uint32 flags);
	void close_disk(int drv);
	int  change_disk(int drv);
	void set_disk_side(int drv, int side);
	int  get_disk_side(int drv);
	bool disk_inserted(int drv);
	void set_drive_type(int drv, uint8 type);
	uint8 get_drive_type(int drv);
	uint8 fdc_status();
	void toggle_disk_write_protect(int drv);
	bool disk_write_protected(int drv);

	uint16 get_drive_select();

	void mix(int32* buffer, int cnt);
	void set_volume(int vol);
	void init(int rate, int volume);

	void save_state(FILEIO *fp);
	void load_state(FILEIO *fp);
};

#endif

